(function(e){function n(n){for(var t,c,a=n[0],l=n[1],u=n[2],f=0,p=[];f<a.length;f++)c=a[f],o[c]&&p.push(o[c][0]),o[c]=0;for(t in l)Object.prototype.hasOwnProperty.call(l,t)&&(e[t]=l[t]);s&&s(n);while(p.length)p.shift()();return i.push.apply(i,u||[]),r()}function r(){for(var e,n=0;n<i.length;n++){for(var r=i[n],t=!0,a=1;a<r.length;a++){var l=r[a];0!==o[l]&&(t=!1)}t&&(i.splice(n--,1),e=c(c.s=r[0]))}return e}var t={},o={app:0},i=[];function c(n){if(t[n])return t[n].exports;var r=t[n]={i:n,l:!1,exports:{}};return e[n].call(r.exports,r,r.exports,c),r.l=!0,r.exports}c.m=e,c.c=t,c.d=function(e,n,r){c.o(e,n)||Object.defineProperty(e,n,{enumerable:!0,get:r})},c.r=function(e){"undefined"!==typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})},c.t=function(e,n){if(1&n&&(e=c(e)),8&n)return e;if(4&n&&"object"===typeof e&&e&&e.__esModule)return e;var r=Object.create(null);if(c.r(r),Object.defineProperty(r,"default",{enumerable:!0,value:e}),2&n&&"string"!=typeof e)for(var t in e)c.d(r,t,function(n){return e[n]}.bind(null,t));return r},c.n=function(e){var n=e&&e.__esModule?function(){return e["default"]}:function(){return e};return c.d(n,"a",n),n},c.o=function(e,n){return Object.prototype.hasOwnProperty.call(e,n)},c.p="/deck/";var a=window["webpackJsonp"]=window["webpackJsonp"]||[],l=a.push.bind(a);a.push=n,a=a.slice();for(var u=0;u<a.length;u++)n(a[u]);var s=l;i.push([0,"chunk-vendors"]),r()})({0:function(e,n,r){e.exports=r("56d7")},"56d7":function(e,n,r){"use strict";r.r(n);r("cadf"),r("551c"),r("f751"),r("097d");var t=r("2b0e"),o=function(){var e=this,n=e.$createElement,r=e._self._c||n;return r("div",{attrs:{id:"app"}},[r("router-view")],1)},i=[],c=r("2877"),a={},l=Object(c["a"])(a,o,i,!1,null,null,null),u=l.exports,s=r("8c4f"),f=function(){var e=this,n=e.$createElement,r=e._self._c||n;return r("div",{staticClass:"home"},[r("HelloWorld")],1)},p=[],d=function(){var e=this,n=e.$createElement;e._self._c;return e._m(0)},v=[function(){var e=this,n=e.$createElement,r=e._self._c||n;return r("div",[r("h1",[e._v("Deck is empty")]),r("a",{attrs:{href:"/admin/home"}},[e._v("Ax admin")])])}],h={name:"HelloWorld",props:{msg:String}},m=h,b=Object(c["a"])(m,d,v,!1,null,"5782dba6",null),g=b.exports,w={name:"home",components:{HelloWorld:g}},y=w,_=Object(c["a"])(y,f,p,!1,null,null,null),j=_.exports;t["a"].use(s["a"]);var O=new s["a"]({mode:"history",base:"/deck/",routes:[{path:"/",name:"home",component:j}]}),k=r("2f62");t["a"].use(k["a"]);var x=new k["a"].Store({state:{},mutations:{},actions:{}}),S=r("9483");Object(S["a"])("".concat("/deck/","service-worker.js"),{ready:function(){console.log("App is being served from cache by a service worker.\nFor more details, visit https://goo.gl/AFskqB")},registered:function(){console.log("Service worker has been registered.")},cached:function(){console.log("Content has been cached for offline use.")},updatefound:function(){console.log("New content is downloading.")},updated:function(){console.log("New content is available; please refresh.")},offline:function(){console.log("No internet connection found. App is running in offline mode.")},error:function(e){console.error("Error during service worker registration:",e)}}),t["a"].config.productionTip=!1,new t["a"]({router:O,store:x,render:function(e){return e(u)}}).$mount("#app")}});
//# sourceMappingURL=app.0765a504.js.map