(window.webpackJsonp=window.webpackJsonp||[]).push([["chunk-2d22bdd8"],{f190:function(n,e,t){"use strict";t.r(e);var o={name:"AxChangelogColumn",data:function(){return{num:null}},mounted:function(){},methods:{}},u=t("2877"),a=Object(u.a)(o,function(){var n=this,e=n.$createElement;return(n._self._c||e)("span",[n._v("\n  "+n._s(n.i18n.t("common.no-column",{name:"AxChangelog"}))+"\n  "),n._t("default")],2)},[],!1,null,"1cc82742",null);e.default=a.exports}}]);