self.__precacheManifest = [
  {
    "revision": "72869256053439078751",
    "url": "/deck/static/js/app.0765a504.js"
  },
  {
    "revision": "7009cf92910b0dd61b49",
    "url": "/deck/static/js/chunk-vendors.5de009e4.js"
  },
  {
    "revision": "77f5bd0593089704adf980d89e925bc6",
    "url": "/deck/index.html"
  },
  {
    "revision": "b6216d61c03e6ce0c9aea6ca7808f7ca",
    "url": "/deck/robots.txt"
  }
];