(window.webpackJsonp=window.webpackJsonp||[]).push([["chunk-5589616c"],{"16bc":function(e,n,t){(e.exports=t("24fb")(!1)).push([e.i,".changelog-row[data-v-8dec9150]{padding-top:5px}",""])},"6eef":function(e,n,t){"use strict";var a=t("d15c");t.n(a).a},a720:function(e,n,t){"use strict";t.r(n);var a=(t("7f7f"),t("ac6a"),{name:"AxChangelog",props:{name:null,dbName:null,tag:null,options:null,value:null,isRequired:null},data:function(){return{currentValue:null,errors:[]}},watch:{value:function(e){this.currentValue=e,e||(this.currentValue=[])}},created:function(){this.currentValue=this.value,this.value||(this.currentValue=[])},methods:{isValid:function(){return!0},getDate:function(e){return new Date(1e3*e)},getModifiedFields:function(e){var n=[];return e.changed_fields.forEach(function(e){n.push(e.name)}),n.length>0?n.join(", "):'<i class="far fa-window-close"></i>'}}}),r=(t("6eef"),t("2877")),s=Object(r.a)(a,function(){var e=this,n=e.$createElement,t=e._self._c||n;return t("div",[t("h3",[e._v(e._s(e.name))]),e._l(e.currentValue,function(n){return t("div",{key:n.timestamp,staticClass:"changelog-row"},[e._v("\n    "+e._s(e.$d(e.getDate(n.timestamp),"normal"))+" : [AnonUser] :\n    "),t("b",[e._v(e._s(n.action.name))]),e._v("\n     \n    "),t("i",{staticClass:"fas fa-arrow-right"}),e._v("\n     \n    "),t("span",{domProps:{innerHTML:e._s(e.getModifiedFields(n))}})])}),0==this.currentValue.length?t("b",[e._v(e._s(e.$t("types.AxChangelog.no-records")))]):e._e()],2)},[],!1,null,"8dec9150",null);n.default=s.exports},d15c:function(e,n,t){var a=t("16bc");"string"==typeof a&&(a=[[e.i,a,""]]),a.locals&&(e.exports=a.locals),(0,t("499e").default)("a4c35770",a,!0,{sourceMap:!1,shadowMode:!1})}}]);