(window.webpackJsonp=window.webpackJsonp||[]).push([["chunk-ae069122"],{a559:function(e,a,t){"use strict";t.r(a);var n=(t("7f7f"),{name:"AxState",props:{name:null,dbName:null,tag:null,optionsJson:null,value:null},computed:{currentName:function(){return this.name?this.name:""}}}),s=(t("fdf1"),t("2877")),l=Object(s.a)(n,function(){var e=this,a=e.$createElement,t=e._self._c||a;return t("div",[t("span",{staticClass:"label"},[e._v(e._s(e.currentName))]),t("div",{staticClass:"value-div"},[e._v(e._s(e.value))]),t("hr")])},[],!1,null,"75044d86",null);a.default=l.exports},c36a:function(e,a,t){var n=t("e8f3");"string"==typeof n&&(n=[[e.i,n,""]]),n.locals&&(e.exports=n.locals),(0,t("499e").default)("52becb38",n,!0,{sourceMap:!1,shadowMode:!1})},e8f3:function(e,a,t){(e.exports=t("24fb")(!1)).push([e.i,".label[data-v-75044d86]{font-size:13px;color:#000;opacity:.54}.value-div[data-v-75044d86]{min-height:26px;font-size:16px;font-weight:700}",""])},fdf1:function(e,a,t){"use strict";var n=t("c36a");t.n(n).a}}]);